/**
 * DEB2DK.java
 * 
 * @author Marlon A. Braun
 * @version 1.0
 */
package jmetal.problems;

import jmetal.base.Problem;
import jmetal.base.Solution;
import jmetal.base.solutionType.ArrayRealSolutionType;
import jmetal.base.solutionType.BinaryRealSolutionType;
import jmetal.base.solutionType.RealSolutionType;
import jmetal.util.JMException;
import jmetal.util.wrapper.XReal;

/**
 * This class represents the DO2DK problem
 *
 */
public class DEB2DK extends Problem {
	
	/**
	 * Parameter that controls the number of knees
	 */
	private double k;
	
	/**
	 * Default constructur, that creates an instance of DO2DK using "Real" encoding of decision variables
	 * @param nVar Number of decision variables
	 * @param k Parameter that controls the number of knees
	 * @throws ClassNotFoundException
	 */
	public DEB2DK(int nVar, int k) throws ClassNotFoundException {
		this("Real" , nVar , k);
	}
	
	/**
	 * Constructur, that creates an instance of DEB2DK using "Real" encoding of decision variables
	 * @param solutionType solutionType The solution type must "Real", "BinaryReal, and "ArrayReal"
	 * @param k Parameter that controls the number of knees
	 * @throws ClassNotFoundException
	 */
	public DEB2DK(String solutionType, int nVar, int k) throws ClassNotFoundException {
		this.k = (double) k;
		
		numberOfVariables_   = nVar		;
		numberOfObjectives_  = 2		;
		numberOfConstraints_ = 0	 	;
		problemName_ 		 = "DEB2DK"	;
		
		upperLimit_ = new double[numberOfVariables_] ;
	    lowerLimit_ = new double[numberOfVariables_] ;
	    
	    for (int i = 0; i < numberOfVariables_; i++) {
	        lowerLimit_[i] = 0.0 ;
	        upperLimit_[i] = 1.0  ;
	      } // for
		
	    if (solutionType.compareTo("BinaryReal") == 0)
	    	solutionType_ = new BinaryRealSolutionType(this) ;
	    else if (solutionType.compareTo("Real") == 0)
	    	solutionType_ = new RealSolutionType(this) ;
	    else if (solutionType.compareTo("ArrayReal") == 0)
	    	solutionType_ = new ArrayRealSolutionType(this) ;
	    else {
	    	System.out.println("Error: solution type " + solutionType + " invalid") ;
	    	System.exit(-1) ;
	    }
	}
	
	/**
	 * Evaluates a solution of DEB2DK
	 */
	public void evaluate(Solution solution) throws JMException {
		XReal xreal = new XReal(solution) ;
        
		// Store decision variables in array
		double[] x = new double[numberOfVariables_];
		for(int i = 0 ; i < numberOfVariables_ ; i++){
			x[i] = xreal.getValue(i);
		}
		
		// Calculate the sum necessary for computing g (sum of x_2 till x_n)
		double sum = 0;
		for(int i = 1 ; i < numberOfVariables_ ; i++){
			sum += x[i];
		}
		
		double g = 1 + 9/(numberOfVariables_ - 1)*sum;
		double r = 5 + 10*Math.pow( x[0] - 0.5 , 2) + 1 / k * Math.cos(2 * k * Math.PI * x[0]);
		
		solution.setObjective(0 , g*r*Math.sin(Math.PI * x[0] / 2));
		solution.setObjective(1 , g*r*Math.cos(Math.PI * x[0] / 2));

	} // evaluate

}