/**
 * DTLZ8.java
 *
 * @author Marlon A. Braun
 * @version 1.0
 */
package jmetal.problems.DTLZ;

import jmetal.base.Problem;
import jmetal.base.Solution;
import jmetal.base.solutionType.ArrayRealSolutionType;
import jmetal.base.solutionType.BinaryRealSolutionType;
import jmetal.base.solutionType.RealSolutionType;
import jmetal.util.JMException;
import jmetal.util.wrapper.XReal;

/**
 * This class represents the DTLZ8 problem
 */
public class DTLZ8 extends Problem {
	
	/**
	 * Creates a default 3-dimensional instance of DTLZ8
	 * @param solutionType 
	 * @throws ClassNotFoundException
	 */
	public DTLZ8(String solutionType) throws ClassNotFoundException{
		this(solutionType, 3, 30);
	}
	
	/**
	 * Creates a default instance of DTZLZ8 using "Real" encoding of variables
	 * 
	 * @param nObj Number of objectives
	 * @param nReal Number of (real) variables
	 * @throws ClassNotFoundException
	 */
	public DTLZ8(int nObj, int nReal) throws ClassNotFoundException{
		this("Real", nObj , nReal);
	}
	
	/**
	 * Creates an instance of DTZLZ8
	 * 
	 * @param solutionType The solution type must "Real", "BinaryReal, and "ArrayReal"
	 * @param nObj Number of objectives
	 * @param nReal Number of (real) variables
	 * @throws ClassNotFoundException
	 */
	public DTLZ8(String solutionType , int nObj, int nReal) throws ClassNotFoundException {
		numberOfVariables_   = nReal  	;
		numberOfObjectives_  = nObj	  	;
		numberOfConstraints_ = nObj		;
		problemName_ 		 = "DTLZ8"	;
		
		upperLimit_ = new double[numberOfVariables_] ;
	    lowerLimit_ = new double[numberOfVariables_] ;
	    
	    for (int i = 0; i < numberOfVariables_; i++) {
	        lowerLimit_[i] = 0.0 ;
	        upperLimit_[i] = 1.0  ;
	      } // for
	    
	    if (solutionType.compareTo("BinaryReal") == 0)
	    	solutionType_ = new BinaryRealSolutionType(this) ;
	    else if (solutionType.compareTo("Real") == 0)
	    	solutionType_ = new RealSolutionType(this) ;
	    else if (solutionType.compareTo("ArrayReal") == 0)
	    	solutionType_ = new ArrayRealSolutionType(this) ;
	    else {
	    	System.out.println("Error: solution type " + solutionType + " invalid") ;
	    	System.exit(-1) ;
	    }
	} // DTLZ8
	
	/** 
	  * Evaluates a solution.
	  * @param solution The solution to evaluate
	  * @throws JMException 
	  */
	public void evaluate(Solution solution) throws JMException {
		
		int nobj = this.getNumberOfObjectives();
		int nreal = this.getNumberOfVariables();
		
		XReal x = new XReal(solution);
		double[] xreal = new double[nreal];
		double[] obj = new double[nobj];
		
		for(int i = 0 ; i < nreal ; i++){
			xreal[i] = x.getValue(i);
		}
		
		double f, g;
	    int i, j, low, up;
	    for (i = 1; i <= nobj; ++i) {
	    	f = 0.0;
	    	low = (int) Math.floor((i-1)*nreal/nobj) + 1;
	    	up = (int) Math.floor(i*nreal/nobj);
	    	for (j = low; j <= up ; ++j) {
				f += xreal[j-1];
			}
			obj[i-1] = 1 / Math.floor(nreal/nobj) * f;
	    }
	    
	    for(int k = 0 ; k < this.getNumberOfObjectives() ; k++){
	    	solution.setObjective(k, obj[k]);
	    }
	} //evaluate
	
	/**
	 * Evalutes the number of constraints.
	 * 
	 * @param solution
	 */
	public void evaluateConstraints(Solution solution) throws JMException{
		
		int nobj = this.getNumberOfObjectives();
		
		double[] obj = new double[nobj];
		double[] constr = new double[numberOfConstraints_];
		
		for(int i = 0 ; i < nobj ; i++){
			obj[i] = solution.getObjective(i);
		}
		
		double g;
	    int i, j;
	    for (i = 1; i < nobj; ++i) {
	    	g = obj[nobj-1] + 4 * obj[i-1] - 1;
	    	constr[i-1] = g;
		}
	    g = obj[0] + obj[1];
	    for (i = 1; i < nobj-1; ++i) {
	    	for (j = i+1; j <= nobj-1; ++j){
	    		if ((obj[i-1] + obj[j-1]) < g){
	    			g = obj[i-1] + obj[j-1];
	    		}
	    	}
	    }
	    constr[nobj-1] = 2 * obj[nobj-1] + g - 1;
	    
	    double total = 0.0;
	    int number = 0;
	    for (int k = 0; k < numberOfConstraints_; k++) {
	        if (constr[k] < 0.0){
	          total+=constr[k];
	          number++;
	        }
	      }
	    
	    solution.setOverallConstraintViolation(total);    
	    solution.setNumberOfViolatedConstraint(number);
	} //evaluateConstraints
}
