/**
 * MCrowdingArchive.java
 *
 * @author Juan J. Durillo
 * @author Marlon A. Braun
 * @version 1.1
 * 
 * This CrowdingArchive has been modified to use M-Proper Pareto optimality
 * each time a <code>Solution</code> is added to the archive.
 */
package jmetal.util.archive;

import jmetal.base.*;
import jmetal.base.operator.comparator.*;
import java.util.Comparator;
import java.util.*;
import jmetal.util.Distance;
import jmetal.util.MProperParetoOptimality;

/**
 * This class implements a bounded archive based on crowding distances (as
 * defined in NSGA-II).
 * 
 * It has been modified to include M-proper Pareto optimality.
 */
public class MCrowdingArchive extends Archive {
  
  /** 
   * Stores the maximum size of the archive.
   */
  private int maxSize_;
  
  /**
   * stores the number of the objectives.
   */
  private int objectives_;    
  
  /**
   * Stores a <code>Comparator</code> for dominance checking.
   */
  private Comparator dominance_;
  
  /**
   * Stores a <code>Comparator</code> for equality checking (in the objective
   * space).
   */
  private Comparator equals_; 

  /**
   * Stores a <code>Comparator</code> for checking crowding distances.
   */
  private Comparator crowdingDistance_; 
  
  /**
   * Stores a <code>Distance</code> object, for distances utilities
   */
  private Distance distance_; 
  
  /**
   * Stores the number, which is used for M-proper Pareto Optimality
   */
  private double M;
  
  /**
   * The object, with which M-proper Pareto optimality sorting is done
   */
  private MProperParetoOptimality mppo;
  
  /**
   * Constructor. 
   * @param maxSize The maximum size of the archive.
   * @param numberOfObjectives The number of objectives.
   */
  public MCrowdingArchive(int maxSize, int numberOfObjectives, double M) {
    super(maxSize);
    maxSize_          = maxSize;
    objectives_       = numberOfObjectives;        
    dominance_        = new DominanceComparator();
    equals_           = new EqualSolutions();
    crowdingDistance_ = new CrowdingDistanceComparator();
    distance_         = new Distance();
    this.M 			  = M;
    mppo 			  = new MProperParetoOptimality( M );
    
  } // CrowdingArchive
    
  
  /**
   * Adds a <code>Solution</code> to the archive. If the <code>Solution</code>
   * is dominated by any member of the archive, then it is discarded. If the 
   * <code>Solution</code> dominates some members of the archive, these are
   * removed. If the archive is full and the <code>Solution</code> has to be
   * inserted, the solutions are sorted by crowding distance and the one having
   * the minimum crowding distance value.
   * @param solution The <code>Solution</code>
   * @return true if the <code>Solution</code> has been inserted, false 
   * otherwise.
   * 
   * This method has been modified to update the archive using M-proper Pareto
   * optimality each time a <code>Solution</code> is added to the archive
   */
  public boolean add(Solution solution){
    int flag = 0;
    int i = 0;
    Solution aux; //Store an solution temporally
    while (i < solutionsList_.size()){
      aux = solutionsList_.get(i);            
            
      flag = dominance_.compare(solution,aux);
      if (flag == 1) {               // The solution to add is dominated
        return false;                // Discard the new solution
      } else if (flag == -1) {       // A solution in the archive is dominated
        solutionsList_.remove(i);    // Remove it from the population            
      } else {
          if (equals_.compare(aux,solution)==0) { // There is an equal solution 
        	                                      // in the population
            return false; // Discard the new solution
          }  // if
          i++;
      }
    }
    
    // MPPO SORTING INCLUDED HERE
    SolutionSet results[] = mppo.create(this);
    if(results[0].size() != 0 && results[1].size() != 0){
    	this.clear();
        Iterator it = results[0].iterator();
        while(it.hasNext()){
        	solutionsList_.add( (Solution) it.next());
        }
    }
    
    // Insert the solution into the archive
    solutionsList_.add(solution);        
    if (size() > maxSize_) { // The archive is full
      distance_.crowdingDistanceAssignment(this,objectives_);
      sort(crowdingDistance_);
      //Remove the last
      remove(maxSize_);
    }        
    return true;
  } // add
} // CrowdingArchive
